@@ -4,6 +4,7 @@ from loguru import logger
 import traceback
 from time import sleep
 from typing import Union
+import queue
 
 
 class base_tredis_msg_sender:
@@ -22,11 +23,50 @@ class base_tredis_msg_sender:
         return None
 
 
-class Tredis(threading.Thread):
-    default_port = 6379
-    default_db = 0
+class Tredis_publish(threading.Thread):
+    str_exit_magic_words = 'Exit Exit Exit now'
+
+    def __init__(self,
+                 server='localhost',
+                 port=6379,
+                 db=0,
+                 password=''):
+        threading.Thread.__init__(self)
+        self.server = server
+        self.port = port
+        self.db = db
+        self.password = password
+        self.queue_publish = queue.Queue()
+
+        self.r = redis.StrictRedis(host=self.server,
+                                   port=self.port,
+                                   db=self.db,
+                                   charset="utf-8",
+                                   decode_responses=True,
+                                   password=self.password)
+
+        self.start()
+
+    def run(self):
+        while True:
+            try:
+                channel, message = self.queue_publish.get(timeout=1)
+            except queue.Empty:
+                pass
+            else:
+                self.r.publish(channel, message)
+                self.queue_publish.task_done()
+                if channel == Tredis_publish.str_exit_magic_words and message == Tredis_publish.str_exit_magic_words:
+                    break
+
+    def publish(self, channel, message):
+        self.queue_publish.put((channel, message), block=True)
+
+    def stop(self):
+        self.publish(Tredis_publish.str_exit_magic_words, Tredis_publish.str_exit_magic_words)
 
 
+class Tredis_subscribe(threading.Thread):
     def __init__(self,
                  server='localhost',
                  port=6379,
@@ -63,12 +103,12 @@ class Tredis(threading.Thread):
         logger.info(f'Redis subscribe to channel [{self.channel}]')
 
         self.start()
+        sleep(1)
 
     def subscribe(self, channel):
         logger.info(f'Redis subscribe to extra channel [{channel}]')
         self.sub.subscribe(channel)
 
-
     def run(self):
         t = threading.current_thread()
         self.sub.subscribe(self.channel)
@@ -81,7 +121,7 @@ class Tredis(threading.Thread):
                     logger.info(f'redis_working_thread got msg: {data}')
 
                     if isinstance(data, str) and data.startswith(self.str_thread_exit_magic):
-                        if data[len(self.str_thread_exit_magic)+1:] == str(self):
+                        if data[len(self.str_thread_exit_magic) + 1:] == str(self):
                             logger.info(f'{self} to exit')
                             break
                     else:
@@ -100,30 +140,64 @@ class Tredis(threading.Thread):
         self.r.publish(self.channel, f'{self.str_thread_exit_magic} {str(self)}')
 
 
+class Tredis:
+    def __init__(self,
+                 server='localhost',
+                 port=6379,
+                 db=0,
+                 password='',
+                 channel='test',
+                 prefix='test',
+                 redis_msg_sender=base_tredis_msg_sender()):
+        self.tredis_subscribe = Tredis_subscribe(server, port, db, password, channel, prefix, redis_msg_sender)
+        self.tredis_publish = Tredis_publish(server, port, db, password)
+
+        self.r = self.tredis_publish.r
+
+    def subscribe(self, channel):
+        self.tredis_subscribe.subscribe(channel)
+
+    def publish(self, channel, message):
+        self.tredis_publish.publish(channel, message)
+
+    def stop(self):
+        self.tredis_subscribe.stop()
+        self.tredis_subscribe.join()
+        self.tredis_publish.stop()
+        self.tredis_publish.join()
+
+
+
 
 if __name__ == '__main__':
+    class warrant_channel:
+        all = 'warrant_all'
+        dealer = 'warrant_dealer'
+        large_volume = 'warrant_large_volume'
+        burst = 'warrant_burst'
+
+
     logger.disable('mypylib.tredis')
     tredis = Tredis()
 
-    tredis.subscribe('test1')
-    tredis.subscribe('test2')
+    tredis.subscribe(warrant_channel.all)
+    tredis.subscribe(warrant_channel.dealer)
+    tredis.subscribe(warrant_channel.large_volume)
+    tredis.subscribe(warrant_channel.burst)
 
-    tredis.r.publish('test', 'This is a book')
-    tredis.r.publish('test1', 'That is a pencil.')
-    tredis.r.publish('test2', 'My name is John.')
+    tredis.publish(warrant_channel.all, warrant_channel.all)
+    tredis.publish(warrant_channel.dealer, warrant_channel.dealer)
+    tredis.publish(warrant_channel.large_volume, warrant_channel.large_volume)
+    tredis.publish(warrant_channel.burst, warrant_channel.burst)
 
     index = 0
     while True:
         try:
             sleep(1)
             index += 1
-            if index == 10000000000:
+            if index == 3:
                 break
         except KeyboardInterrupt:
             break
 
     tredis.stop()
-
-
-
-
